package com.sun.corba.se.impl.oa.poa;

import java.util.EmptyStackException;
import org.omg.PortableServer.POAPackage.WrongPolicy;
import org.omg.PortableServer.POAPackage.ServantNotActive;
import org.omg.PortableServer.POA;
import org.omg.CORBA.Object;
import org.omg.PortableServer.Servant;
import com.sun.corba.se.impl.logging.POASystemException;
import com.sun.corba.se.spi.orb.ORB;
import org.omg.PortableServer.portable.Delegate;

public class DelegateImpl implements Delegate
{
    private ORB orb;
    private POASystemException wrapper;
    private POAFactory factory;
    
    public DelegateImpl(ORB orb, POAFactory factory) {
        this.orb = orb;
        this.wrapper = POASystemException.get(orb, "oa");
        this.factory = factory;
    }
    
    public org.omg.CORBA.ORB orb(Servant servant) {
        return this.orb;
    }
    
    public org.omg.CORBA.Object this_object(Servant servant) {
        try {
            byte[] id = this.orb.peekInvocationInfo().id();
            POA poa = (POA)this.orb.peekInvocationInfo().oa();
            return poa.create_reference_with_id(id, servant._all_interfaces(poa, id)[0]);
        }
        catch (EmptyStackException ex) {
            POAImpl poaImpl;
            try {
                poaImpl = (POAImpl)servant._default_POA();
            }
            catch (ClassCastException ex2) {
                throw this.wrapper.defaultPoaNotPoaimpl(ex2);
            }
            try {
                if (poaImpl.getPolicies().isImplicitlyActivated() || (poaImpl.getPolicies().isUniqueIds() && poaImpl.getPolicies().retainServants())) {
                    return poaImpl.servant_to_reference(servant);
                }
                throw this.wrapper.wrongPoliciesForThisObject();
            }
            catch (ServantNotActive servantNotActive) {
                throw this.wrapper.thisObjectServantNotActive(servantNotActive);
            }
            catch (WrongPolicy wrongPolicy) {
                throw this.wrapper.thisObjectWrongPolicy(wrongPolicy);
            }
        }
        catch (ClassCastException ex3) {
            throw this.wrapper.defaultPoaNotPoaimpl(ex3);
        }
    }
    
    public POA poa(Servant servant) {
        try {
            return (POA)this.orb.peekInvocationInfo().oa();
        }
        catch (EmptyStackException ex) {
            POA lookupPOA = this.factory.lookupPOA(servant);
            if (lookupPOA != null) {
                return lookupPOA;
            }
            throw this.wrapper.noContext(ex);
        }
    }
    
    public byte[] object_id(Servant servant) {
        try {
            return this.orb.peekInvocationInfo().id();
        }
        catch (EmptyStackException ex) {
            throw this.wrapper.noContext(ex);
        }
    }
    
    public POA default_POA(Servant servant) {
        return this.factory.getRootPOA();
    }
    
    public boolean is_a(Servant servant, String s) {
        String[] all_interfaces = servant._all_interfaces(this.poa(servant), this.object_id(servant));
        int i = 0;
        while (i < all_interfaces.length) {
            if (s.equals(all_interfaces[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public boolean non_existent(Servant servant) {
        try {
            return this.orb.peekInvocationInfo().id() == null;
        }
        catch (EmptyStackException ex) {
            throw this.wrapper.noContext(ex);
        }
    }
    
    public org.omg.CORBA.Object get_interface_def(Servant servant) {
        throw this.wrapper.methodNotImplemented();
    }
}
